class TTTBoard {
/**
 *   Tic-Tac-Toe game board
 *
 *   Georgy Gimel'farb 20.07.1998
 *   (last modified: 15.09.98)
 */


  private TTTBox [] tttBoard;  // Array of TTTBox-es to form 3x3 game board

  public TTTBoard() {            // Constructor of an instance of the board
    tttBoard = new TTTBox[ 9 ];
    for( int i = 0; i<9; i++ )
         tttBoard[ i ] = new TTTBox();
  }

  // check if the box in position pos is unmarked
  public boolean isEmpty( int pos ) {
    return ( tttBoard [ pos ].isEmpty() );
  }

  // check if there are unmarked positions in the board
  public boolean noFree() {
    boolean noFree = true;
    for( int pos = 0; pos < 9; pos++ )
      noFree &= ( !isEmpty( pos ) );    
    return noFree;     
  }

  // mark O the box in position pos
  public void putO( int pos ) {
    tttBoard[ pos ].putO();
  }

  // mark X the box in position pos
  public void putX( int pos ) {
    tttBoard[ pos ].putX();
  }

  // check if the a winning "three-in-a-line" position exists
  public boolean hit() {
    boolean hit = singleHit( 0, 4, 8 ) | singleHit( 2, 4, 6 );
    for( int pos = 0; pos < 9; pos+=3 )
      hit |= singleHit( pos, pos + 1, pos + 2 );
    for( int pos = 0; pos < 3; pos++ )
      hit |= singleHit( pos, pos + 3, pos + 6 );
    return hit;   
  }

  // check if given three positions have the same O or X mark
  private boolean singleHit( int p1, int p2, int p3 ){
    return ( !isEmpty( p1 ) ) & 
           ( tttBoard[ p1 ].getMark() == tttBoard[ p2 ].getMark() ) &
           ( tttBoard[ p1 ].getMark() == tttBoard[ p3 ].getMark() ) ;
  }

  // convert the game board into a String for printing
  public String toString() {
   String border = "\n+---+---+---+";
   String output = border;
   for(int d = 0; d < 9; d+=3 ) {
     output+= "\n|";
     for( int i = 0; i<3; i++ )
       output += 
         (" " + tttBoard[ i+d ].toString() + " |"); 
     output += border;
   }
   return (output + "\n\n");   
  }
}


